--[[
	Author: another
	
	Custom flyout buttons support
]]
	
local ARROW_BUTTON_DEF_HEIGHT, ARROW_BUTTON_MIN_HEIGHT, ARROW_BUTTON_MAX_HEIGHT = 15, 5, 36
local BF_SETTINGS_GROUP = "Button Facade"


--variables
local AddonName, AddonTable = ...
FbcButtonFrames = {}
FbcArrowButtons = {}

FbcSettingsMode = nil
FbcKeybindMode = nil
FbcArrowButtonsHeight = nil
FbcShowBorders = nil
FbcEnableMouseover = nil
FbcHideOnClick = nil
FbcStubFrame = CreateFrame("Frame", nil, UIParent, "SecureHandlerBaseTemplate")

local EventHandlersTable = {}
local EnteringWorldOrVariablesLoaded = false
--Button Facade stuff
local LBF
local BF_Table = {}
FbcLBFMasterGroup = nil
			


--[[structures
ActionButton
	- customFlyout
	ArrowBtn
		- FlyoutArrow		
		- expandDir: up, left, down, right
		- expanded
	
		- FlyoutListFrame	--also through SetFrameRef
			- ButtonList
]]

local function RegisterFrames()
	FbcButtonFrames = {}
	local frame = EnumerateFrames()
	while frame do
		if frame.IsProtected and frame.GetObjectType and frame.GetScript and frame:GetObjectType() == "CheckButton" and frame:IsProtected() then
			local script = frame:GetScript("OnClick")
			if script == ActionButton1:GetScript("OnClick") or tostring(frame:GetName()):match("^BT4Button%d+$") then
				tinsert(FbcButtonFrames, frame)
			end
		end
		frame = EnumerateFrames(frame)
	end
end

local function AttachToAllActionButtons()
	for _, frame in ipairs(FbcButtonFrames) do
		if frame.action or frame._state_action then
			FlyoutArrowButton_Attach(frame)
		end
	end
end

local function ShowAllFlyoutButtons()
	for _, frame in ipairs(FbcButtonFrames) do
		if frame.action or frame._state_action then
			local arrowBtn = FbcArrowButtons[frame:GetName()]
			if arrowBtn then 
				arrowBtn:ShowArrow()
			end
		end
	end
end

local function HideUnusedFlyoutButtons()
	for _, frame in ipairs(FbcButtonFrames) do
		if frame.action or frame._state_action then
			local arrowBtn = FbcArrowButtons[frame:GetName()]
			if arrowBtn then 
				arrowBtn:UpdateArrow()
				arrowBtn:HideArrow()
			end
		end
	end
end

local function HideFlyoutFrames()
	for _, frame in ipairs(FbcButtonFrames) do
		if frame.action or frame._state_action then
			if frame and not(InCombatLockdown()) then
				local arrowBtn = FbcArrowButtons[frame:GetName()]
				if (arrowBtn) then 
					arrowBtn:SetAttribute("expanded", false)
					arrowBtn.FlyoutListFrame:Hide()
				end
			end
		end
	end
end


--[[ Button Facade stuff ]]

function FlyoutButtonCustom_ButtonFacadeCallback(self, SkinID, Gloss, Backdrop, Group, Button, Colors)
	-- If no group is specified, save the data as the root add-on skin.
	-- This will allow the ButtonFacade GUI to display it correctly.
	if not(Group) then
		FlyoutButtonCustom_Settings[BF_SETTINGS_GROUP]["SkinID"] = SkinID
		FlyoutButtonCustom_Settings[BF_SETTINGS_GROUP]["Gloss"] = Gloss
		FlyoutButtonCustom_Settings[BF_SETTINGS_GROUP]["Backdrop"] = Backdrop
		FlyoutButtonCustom_Settings[BF_SETTINGS_GROUP]["Colors"] = Colors
	else
		FlyoutButtonCustom_Settings[BF_SETTINGS_GROUP][Group]["SkinID"] = SkinID
		FlyoutButtonCustom_Settings[BF_SETTINGS_GROUP][Group]["Gloss"] = Gloss
		FlyoutButtonCustom_Settings[BF_SETTINGS_GROUP][Group]["Backdrop"] = Backdrop
		FlyoutButtonCustom_Settings[BF_SETTINGS_GROUP][Group]["Colors"] = Colors
	end
end


--[[ events stuff ]]

local EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
EventFrame:RegisterEvent("VARIABLES_LOADED")

function EventFrame:LIBKEYBOUND_ENABLED()
	FbcKeybindMode = true
end

function EventFrame:LIBKEYBOUND_DISABLED()
	FbcKeybindMode = nil
end

function EventFrame:LIBKEYBOUND_MODE_COLOR_CHANGED()
	print("color changed")
end


--event handlers
local function DoUpdates(func)
	for _, arrowBtn in pairs(FbcArrowButtons) do
		for i, v in ipairs(arrowBtn.FlyoutListFrame.ButtonList) do
			if v.command then
				func(v)
			end
		end
	end
end

local function EventEnteringCombat(self, ...)
	FbcSettingsMode = false
end

local function EventTalentGroupChanged(self, ...)
	if InCombatLockdown() then
		return
	end
	
	for _, arrowBtn in pairs(FbcArrowButtons) do
		for i, v in ipairs(arrowBtn.FlyoutListFrame.ButtonList) do
			v.command, v.value, v.subValue = nil, nil, nil
		end
	end

	HideFlyoutFrames()
	FbcArrowButtons = {}
	RegisterFrames()
	AttachToAllActionButtons()
	
	DoUpdates(FlyoutListButton.SetTooltip)
	DoUpdates(FlyoutListButton.UpdateTexture)
	DoUpdates(FlyoutListButton.UpdateCooldown)
	DoUpdates(FlyoutListButton.UpdateChecked)
	DoUpdates(FlyoutListButton.UpdateEquipped)
	DoUpdates(FlyoutListButton.UpdateUsable)
	DoUpdates(FlyoutListButton.UpdateButtonText)
end

local function EventLearnedSpellInTab(self, ...)
	DoUpdates(FlyoutListButton.SetTooltip)
	DoUpdates(FlyoutListButton.UpdateTexture)
	DoUpdates(FlyoutListButton.UpdateCooldown)
	DoUpdates(FlyoutListButton.UpdateChecked)
	DoUpdates(FlyoutListButton.UpdateUsable)
	DoUpdates(FlyoutListButton.UpdateButtonText)
end

local function RegisterEvents()
	--event table
	EventHandlersTable = {
		--common
		["PLAYER_REGEN_DISABLED"] 			= EventEnteringCombat,
		["ACTIVE_TALENT_GROUP_CHANGED"] 	= EventTalentGroupChanged,
		["LEARNED_SPELL_IN_TAB"] 			= EventLearnedSpellInTab,
	}
	
	--EventFrame:RegisterEvent("ACTIONBAR_HIDEGRID")
	EventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
	EventFrame:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED")
	EventFrame:RegisterEvent("LEARNED_SPELL_IN_TAB")
end

local function FlyoutButtonCustom_SlashHandler(msg, editbox)
	local command, rest = msg:match("^(%S*)%s*(.-)$")
	-- Any leading non-whitespace is captured into command
	-- the rest (minus leading whitespace) is captured into rest.
	command = string.lower(command)
	
	if (command == "" or command == "help") then
		print("FlyoutButton Custom commands:")
		print("'border' to toggle button borders")
		print("'kb' or 'keybind' to toggle keybinding mode")
		print("'mouseover' to toggle opening/hiding lists on arrow button mouseover")
		print("'hide' to toggle hiding on list button click")
		print("'height value' to set arrow button height, values 5-36")
		print("'settings' to toggle SettingsMode")
		print("'unique' to toggle only 1 list visible")
	elseif (command == "border") then
		FbcShowBorders = not(FbcShowBorders)
		FlyoutButtonCustom_Settings["ShowBorders"] = FbcShowBorders
		print("ShowBorders = "..tostring(FbcShowBorders))

		for _, arrowBtn in pairs(FbcArrowButtons) do
			local flf = arrowBtn.FlyoutListFrame
			for i, v in ipairs(arrowBtn.FlyoutListFrame.ButtonList) do
				if v.command then
					v:UpdateTexture()
				end
			end
		end
	elseif (command == "hide") then
		FbcHideOnClick = not(FbcHideOnClick)
		FlyoutButtonCustom_Settings["HideOnClick"] = FbcHideOnClick
		print("HideOnClick = "..tostring(FbcHideOnClick))

		for _, arrowBtn in pairs(FbcArrowButtons) do
			for _, v in ipairs(arrowBtn.FlyoutListFrame.ButtonList) do
				v:SetHideOnClick(FbcHideOnClick)
			end
		end
	elseif (command == "height") then
		if rest ~= "" then
			local temp = tonumber(rest)
			if not(temp) or (temp < ARROW_BUTTON_MIN_HEIGHT) or (temp > ARROW_BUTTON_MAX_HEIGHT) then
				print("new height value '"..rest.."' out of range "..ARROW_BUTTON_MIN_HEIGHT.."-"..ARROW_BUTTON_MAX_HEIGHT)
			else
				FbcArrowButtonsHeight = temp
				FlyoutButtonCustom_Settings["ArrowButtonsHeight"] = FbcArrowButtonsHeight
				
				for _, arrowBtn in pairs(FbcArrowButtons) do
					arrowBtn:SetFrameSize(arrowBtn:GetAttribute("expandDir"))
				end
			end
		end
	elseif (command == "kb" or command == "keybind") then
		FbcLibKeyBound:Toggle()
	elseif (command == "mouseover") then
		if not(InCombatLockdown()) then
			FbcEnableMouseover = not(FbcEnableMouseover)
			FlyoutButtonCustom_Settings["EnableMouseover"] = FbcEnableMouseover
			print("EnableMouseover = "..tostring(FbcEnableMouseover))
			for _, arrowBtn in pairs(FbcArrowButtons) do
				if FbcEnableMouseover then
					arrowBtn:SetAttribute("_onenter", ArrowButtonOnEnterSnippet)
				else
					arrowBtn:SetAttribute("_onenter", "")
				end
			end
		else
			print("can't toggle mouseover while in combat")
		end
	elseif (command == "settings") then
		if not(InCombatLockdown()) then
			FbcSettingsMode = not(FbcSettingsMode)
			print("SettingsMode = "..tostring(FbcSettingsMode))
			if FbcSettingsMode then
				EventTalentGroupChanged()
				ShowAllFlyoutButtons()
			else
				HideUnusedFlyoutButtons()
			end
		else
			print("can't enter SettingsMode while in combat")
		end
	elseif (command == "unique") then
		if not(InCombatLockdown()) then
			FbcStubFrame:SetAttribute("unique-list", not(FbcStubFrame:GetAttribute("unique-list")))
			FlyoutButtonCustom_Settings["UniqueList"] = FbcStubFrame:GetAttribute("unique-list")
			print("unique list = "..tostring(FbcStubFrame:GetAttribute("unique-list")))
		else
			print("can't set while in combat")
		end
	else
		print("unknown command '"..command.."'")
	end
end

local spb_frame = SpellBookFrame
spb_frame:HookScript("OnShow", function()
	if not(InCombatLockdown()) then
		FbcSettingsMode = true
		EventTalentGroupChanged()
		ShowAllFlyoutButtons()
	end
end)
spb_frame:HookScript("OnHide", function()
	FbcSettingsMode = false
	if not(InCombatLockdown()) then
		HideUnusedFlyoutButtons()
	end
end)

local function Init()
	FlyoutButtonCustom_Settings = FlyoutButtonCustom_Settings or {}
	local fbc_s = FlyoutButtonCustom_Settings
	
	--read settings
	FbcArrowButtonsHeight = fbc_s["ArrowButtonsHeight"] or ARROW_BUTTON_DEF_HEIGHT
	if fbc_s["ShowBorders"] ~= nil then 
		FbcShowBorders = fbc_s["ShowBorders"]
	else
		FbcShowBorders = true
	end
	if fbc_s["EnableMouseover"] ~= nil then 
		FbcEnableMouseover = fbc_s["EnableMouseover"]
	else
		FbcEnableMouseover = false
	end
	if fbc_s["HideOnClick"] ~= nil then 
		FbcHideOnClick = fbc_s["HideOnClick"]
	else
		FbcHideOnClick = true
	end
	
	if fbc_s["UniqueList"] == nil then
		fbc_s["UniqueList"] = true
	end
	FbcStubFrame:SetAttribute("unique-list", fbc_s["UniqueList"])
	FlyoutButtonCustom_Settings["expandDir"] = FlyoutButtonCustom_Settings["expandDir"] or {}
	--for 2 talent sets
	FlyoutButtonCustom_Settings["expandDir"][1] = FlyoutButtonCustom_Settings["expandDir"][1] or {}
	FlyoutButtonCustom_Settings["expandDir"][2] = FlyoutButtonCustom_Settings["expandDir"][2] or {}
	
	--Button Facade
	if (LibStub) then
		LBF = LibStub("LibButtonFacade", true)
		if (LBF) then
			FbcLBFMasterGroup = LBF:Group("FlyoutButtonCustom")
			LBF:RegisterSkinCallback("FlyoutButtonCustom", FlyoutButtonCustom_ButtonFacadeCallback, BF_Table)
		end
	end
	FlyoutButtonCustom_Settings[BF_SETTINGS_GROUP] = FlyoutButtonCustom_Settings[BF_SETTINGS_GROUP] or {}
	if (FbcLBFMasterGroup and fbc_s[BF_SETTINGS_GROUP]["SkinID"]) then
		FbcLBFMasterGroup:Skin(fbc_s[BF_SETTINGS_GROUP]["SkinID"], fbc_s[BF_SETTINGS_GROUP]["Gloss"], fbc_s[BF_SETTINGS_GROUP]["Backdrop"], fbc_s[BF_SETTINGS_GROUP]["Colors"])
	end			
	
	RegisterFrames()
	AttachToAllActionButtons()
	RegisterEvents()
	
	--LibKeyBound
	if (LibStub) then
		FbcLibKeyBound = LibStub("LibKeyBound-1.0")
		if (FbcLibKeyBound) then
			FbcLibKeyBound.RegisterCallback(EventFrame, "LIBKEYBOUND_ENABLED")
			FbcLibKeyBound.RegisterCallback(EventFrame, "LIBKEYBOUND_DISABLED")
			FbcLibKeyBound.RegisterCallback(EventFrame, "LIBKEYBOUND_MODE_COLOR_CHANGED")
		end
	end

	SLASH_FBCUSTOM1, SLASH_FBCUSTOM2 = "/fbc", "/fbcustom"
	SlashCmdList["FBCUSTOM"] = FlyoutButtonCustom_SlashHandler
end

local function OnEvent(self, event, ...)
	if EventHandlersTable[event] then
		EventHandlersTable[event](self, ...)
	elseif event == "PLAYER_ENTERING_WORLD" or event == "VARIABLES_LOADED" then
		if EnteringWorldOrVariablesLoaded then
			Init()
		else
			EnteringWorldOrVariablesLoaded = true
		end
		if event == "PLAYER_ENTERING_WORLD" then EventFrame:UnregisterEvent("PLAYER_ENTERING_WORLD") end
		if event == "VARIABLES_LOADED" then EventFrame:UnregisterEvent("VARIABLES_LOADED") end
	end
end

EventFrame:SetScript("OnEvent", OnEvent)
